@@ -504,7 +504,7 @@ void InfluxdbWriter::Flush()
 	request.set(http::field::host, url->GetHost() + ":" + url->GetPort());
 
 	request.body() = body;
-	request.set(http::field::content_length, request.body().size());
+	request.content_length(request.body().size());
 
 	try {
 		if (stream.first) {
